# Copyright (C) 2009 Chris Larson <clarson@kergoth.com>
# Released under the MIT license (see COPYING.MIT for the terms)
#
# gitver.bbclass provides a GITVER variable which is a (fairly) sane version,
# for use in ${PV}, extracted from the ${S} git checkout, assuming it is one.

def git_drop_tag_prefix(version):
    import re
    if re.match("v\d", version):
        return version[1:]
    else:
        return version

# Assume that the Git repository is in ${S} unless the recipe tells us otherwise
GITVER_S ?= "${S}"

GIT_TAGADJUST = "git_drop_tag_prefix(version)"
GITVER = "${@get_git_pv('${GITVER_S}', d, match=None, tagadjust=lambda version:${GIT_TAGADJUST})}"
GITSHA = "${@get_git_hash('${GITVER_S}', d)}"

TAG_MACHINE = "${MACHINE}"
# If we're building the tests, get the tag from the underlying machine instead
TAG_MACHINE_cheetah-tests = "cheetah"
TAG_MACHINE_panther-tests = "panther"
TAG_MACHINE_qemux86 = "panther"

BS_GITVER = "${@get_git_pv('${GITVER_S}', d, match='release/${TAG_MACHINE}/*', tagadjust=brightsign_tagadjust)}"

def brightsign_tagadjust(version):
    import re
    return re.sub(r'^([^/]*/)*', '', version)

def get_git_hash(path, d):
    return bb.process.run(["/usr/bin/git", "rev-parse", "--short", "HEAD"], cwd=path, shell=False)[0].rstrip()

# This checks to see if there are any files in the working
# tree or the index that are different to HEAD. If they are
# then we tack +local on the end of the version to indicate
# that there are local changes. This stops us matching
# anything from the shared sstate cache. We don't make PV
# change based on the mtime of the most recently changed file
# because that would cause us to fetch the source again every
# time which can often be avoided during development. (It
# turns out that "git describe" has this functionality built
# in in via its --dirty flag so we only use this if "git
# describe" fails.)

def get_git_dirty(rootdir):
    try:
        args = [ "(/usr/bin/git diff --quiet && /usr/bin/git diff --cached --quiet) || echo '+local'" ]
        return bb.process.run(args, cwd=rootdir, shell=True)[0].rstrip()
    except Exception, exc:
        # Don't fail in here during recipe parsing
        return "+local-unknown"

def get_git_pv(path, d, match=None, tagadjust=None):
    import os
    import bb.process

    need_machine = d.getVar('COMPATIBLE_MACHINE', True)
    if need_machine:
        import re
        compat_machines = (d.getVar('MACHINEOVERRIDES', True) or "").split(":")
        for m in compat_machines:
            if re.match(need_machine, m):
                break
        else:
            return "0.0"

    rootdir = os.path.abspath(d.getVar("GITVER_S", True))
    try:
        args = [ "/usr/bin/git", "describe", "--tags", "--dirty=+local" ]
        if match:
           args.extend(["--match", match])

        ver = bb.process.run(args, cwd=rootdir, shell=False)[0].rstrip()
    except Exception, exc:
        # Don't fail here during recipe parsing
        bb.warn("get_git_pv %s(%s): %s" % (d.getVar("PN", True), rootdir, exc))
        ver = "0.0-not-in-git"

    if not ver:
        try:
            ver = get_git_hash(rootdir, d) + get_git_dirty(rootdir)
        except Exception, exc:
            bb.fatal(str(exc))

        if ver:
            return "0.0+%s" % ver
        else:
            return "0.0"
    else:
        if tagadjust:
            ver = tagadjust(ver)
        return ver

__BB_DONT_CACHE = "1"
